@@ -100,9 +100,7 @@ def adjust_axes(
         ax.xaxis.set_major_formatter(xfmt)
 
 
-def apply_plot_labels(
-    xlabel: str = "", ylabel: str = "", title: str = "", style: Style = None
-) -> None:
+def apply_plot_labels(xlabel: str = "", ylabel: str = "", title: str = "", style: Style = None) -> None:
     """Helper function to apply labels (xlabel, ylabel, title) according to the style guide (including translation)
 
     :param xlabel: text or rosetta key for the x-axis label
@@ -126,9 +124,7 @@ def apply_plot_labels(
     )
 
 
-def apply_watermark(
-    watermark_text: str, style: Style = None, use_default: bool = True, **kwargs
-) -> None:
+def apply_watermark(watermark_text: str, style: Style = None, use_default: bool = True, **kwargs) -> None:
     """Helper function to apply watermark text to a plot based on config Style object parameters
 
     :param watermark_text: Text to display
@@ -189,6 +185,7 @@ def visualize_1d_distribution(
     legend_strings: Union[Tuple[str], List[str]] = None,
     xlim: Any = None,
     ylim: Any = None,
+    show: bool = False,
     **kwargs,
 ) -> plt.Figure:
     """Core function for visualizing 1-dimensional distribution(s)
@@ -204,6 +201,7 @@ def visualize_1d_distribution(
     :param legend_strings: override legend strings
     :param xlim: optional bound for the x-axis
     :param ylim: optional bound for the y-axis
+    :param show: set to true to trigger plt.show()
     :param kwargs: additional keyword arguments for matplotlib.pyplot.hist()
     :return: figure handle for the plot
     """
@@ -219,9 +217,7 @@ def visualize_1d_distribution(
         data: List[Any] = [data]
 
     # Plot the data and (and best fits if applicable)
-    figure_handle: plt.Figure = plt.figure(
-        facecolor=config.facecolor, figsize=config.figsize
-    )
+    figure_handle: plt.Figure = plt.figure(facecolor=config.facecolor, figsize=config.figsize)
     plt.axes().set_prop_cycle(config.cycler)
     for data_set in data:
         # plotting on log x-axis requires special pre-treatment (log-distributed bin edges)
@@ -230,9 +226,7 @@ def visualize_1d_distribution(
                 if not isinstance(bins, (int, float)):
                     hist_bins = bins
                 else:
-                    hist_bins = np.logspace(
-                        np.log10(min(data_set)), np.log10(max(data_set)), bins
-                    )
+                    hist_bins = np.logspace(np.log10(min(data_set)), np.log10(max(data_set)), bins)
             else:
                 hist_bins = np.logspace(
                     np.log10(min(data_set)),
@@ -260,14 +254,14 @@ def visualize_1d_distribution(
         ylabel_buffer: str = ylabel_translated
     if kwargs.get("density"):
         ylabel_buffer += " (density)"
-    apply_plot_labels(
-        xlabel=xlabel_buffer, ylabel=ylabel_buffer, title=title, style=config
-    )
+    apply_plot_labels(xlabel=xlabel_buffer, ylabel=ylabel_buffer, title=title, style=config)
     if legend_strings:
         plt.legend(legend_strings, fontsize=config.legend_fontsize)
 
     adjust_axes(log_axes=log_axes, style=config, xlim=xlim, ylim=ylim)
     apply_watermark(watermark, style=config)
+    if show:
+        plt.show()
     return figure_handle
 
 
@@ -294,6 +288,7 @@ def visualize_x_y(
     colorbar_label: str = None,
     x_axis_human_tick_labels: bool = False,
     x_axis_formatter: str = "%Y-%m-%d",
+    show: bool = False,
     **kwargs,
 ) -> plt.Figure:
     """Core function for visualizing 2-dimensional data sets
@@ -313,7 +308,6 @@ def visualize_x_y(
     :param plot_best_fit: whether to plot the best fit lines (specify degree, or pass True for degree 1)
     :param xlim: optional bound for the x-axis
     :param ylim: optional bound for the y-axis
-    :param kwargs: additional keyword arguments for matplotlib.pyplot.scatter()
     :param rolling_median_width: window width for rolling average taken by df.y.rolling(rolling_mean_width).mean()
     :param rolling_mean_width: window width for rolling average taken by df.y.rolling(rolling_median_width).median()
     :param colorbar_label: optional label for the colorbar
@@ -321,11 +315,15 @@ def visualize_x_y(
     :param show_colorbar: set to True to show colorbar
     :param x_axis_human_tick_labels: set to True to convert numeric values along the x-axis to human-readable timestamps
     :param x_axis_formatter: format string for the x-axis if human-readable
+    :param show: set to true to trigger plt.show()
+    :param kwargs: additional keyword arguments for matplotlib.pyplot.scatter()
     :return: figure handle for the plot
     """
     # Set style. Overrides: kwargs > style input > Style() defaults
     config: Style = Style(**{**Style().dict(), **make_dict(style), **make_dict(kwargs)})
-    kwargs: Dict[str, Any] = {k: v for k, v in kwargs.items() if k not in config.dict()}
+    kwargs: Dict[str, Any] = {
+        k: v for k, v in kwargs.items() if (k not in config.dict()) or (k == "cmap")
+    }  # allowlisting cmap is a hacky fix until the config handling is refactored
 
     x_data: List[Any] = to_list_if_other_array(x_data)
     y_data: List[Any] = to_list_if_other_array(y_data)
@@ -338,9 +336,7 @@ def visualize_x_y(
         y_data: List[Any] = [y_data]
 
     # Plot the data and (and best fits if applicable)
-    figure_handle: plt.Figure = plt.figure(
-        facecolor=config.facecolor, figsize=config.figsize
-    )
+    figure_handle: plt.Figure = plt.figure(facecolor=config.facecolor, figsize=config.figsize)
     plt.axes().set_prop_cycle(config.cycler)
     scatter_handles: List[Any] = []
 
@@ -356,31 +352,23 @@ def visualize_x_y(
             y_array: np.ndarray = np.asarray(data_set[1])
 
         # Convert to datetime for plotting if intending to use human-readable format
-        if x_axis_human_tick_labels and any(
-            not isinstance(x, datetime.datetime) for x in x_array
-        ):
+        if x_axis_human_tick_labels and any(not isinstance(x, datetime.datetime) for x in x_array):
             if any(np.isnan(x_array)):
                 raise ValueError(
                     "Cannot convert NaN to datetime. Use numeric x-axis labels or filter NaNs upstream."
                 )
-            x_array: List[datetime.datetime] = [
-                datetime.datetime.fromtimestamp(ts) for ts in x_array
-            ]
+            x_array: List[datetime.datetime] = [datetime.datetime.fromtimestamp(ts) for ts in x_array]
 
         if "scatter" in types:
             if log_norm_colors:
                 kwargs.setdefault("norm", matplotlib.colors.LogNorm())
             if kwargs.get("c") is None:
                 kwargs.setdefault("color", config.color)
-            scatter_handles.append(
-                plt.scatter(x_array, y_array, config.markersize, **kwargs)
-            )
+            scatter_handles.append(plt.scatter(x_array, y_array, config.markersize, **kwargs))
 
         if any(x in types for x in ["plot", "line"]):
             includes_line_plot: bool = True
-            p = plt.plot(
-                x_array, y_array, color=config.color, linewidth=config.linewidth
-            )
+            p = plt.plot(x_array, y_array, color=config.color, linewidth=config.linewidth)
 
         # Make sure that the axes facecolor matches the figure facecolor
         plt.gca().set(facecolor=config.facecolor)
@@ -394,9 +382,7 @@ def visualize_x_y(
                 degree: int = plot_best_fit
             else:
                 degree: int = 1
-            plot_best_fit_line(
-                x_array, y_array, degree=degree, color=color, style=config
-            )
+            plot_best_fit_line(x_array, y_array, degree=degree, color=color, style=config)
 
         if rolling_mean_width or rolling_median_width:
             df: pd.DataFrame = pd.DataFrame({"x": x_array, "y": y_array})
@@ -425,9 +411,7 @@ def visualize_x_y(
     ylabel_buffer: str = config.translate(ylabel, missing_response="return_input")
     if "y" in cumulative:
         ylabel_buffer += " (cumulative)"
-    apply_plot_labels(
-        xlabel=xlabel_buffer, ylabel=ylabel_buffer, title=title, style=config
-    )
+    apply_plot_labels(xlabel=xlabel_buffer, ylabel=ylabel_buffer, title=title, style=config)
     if show_colorbar or colorbar_label:
         cbar: plt.colorbar.Colorbar = plt.colorbar()
         if colorbar_label:
@@ -446,6 +430,8 @@ def visualize_x_y(
         x_axis_formatter=x_axis_formatter,
     )
     apply_watermark(watermark, style=config)
+    if show:
+        plt.show()
     return figure_handle
 
 
@@ -465,9 +451,7 @@ def visualize_1d_distribution_interpreter(*args, **kwargs) -> plt.Figure:
         if isinstance(args[0], pd.DataFrame):
             if isinstance(args[1], list):
                 data = [args[0].loc[:, x].tolist() for x in args[1]]
-                kwargs.setdefault(
-                    "legend_strings", [config.translate(x) for x in args[1]]
-                )
+                kwargs.setdefault("legend_strings", [config.translate(x) for x in args[1]])
             elif isinstance(args[1], str):
                 data = args[0].loc[:, args[1]].tolist()
                 kwargs.setdefault("xlabel", config.translate(args[1]))
@@ -546,9 +530,7 @@ def visualize_x_y_input_interpreter(*args, **kwargs) -> plt.Figure:
                     f"Keys {args[1]} & {args[2]} not both in dataframe with: {df.keys().tolist()}"
                 )
         else:
-            raise ValueError(
-                f"Expected positional arguments for feature names, got {args[1]} and {args[2]}"
-            )
+            raise ValueError(f"Expected positional arguments for feature names, got {args[1]} and {args[2]}")
         x_data = df[args[1]]
         y_data = df[args[2]]
         kwargs.setdefault("xlabel", config.translate(args[1]))
@@ -580,6 +562,7 @@ def visualize_hist2d(
     show_colorbar: bool = True,
     colorbar_label: str = "counts",
     zscale: str = "linear",
+    show: bool = False,
     **kwargs,
 ) -> plt.Figure:
     """Visualize a 2-dimensional histogram (basically a heatmap of counts)
@@ -597,23 +580,20 @@ def visualize_hist2d(
     :param zscale: whether the colormap should be 'linear' or 'log'
     :param colorbar_label: text label to place next to the colorbar
     :param show_colorbar: whether to include the colorbar on teh plot
+    :param show: set to true to trigger plt.show()
     :param kwargs: additional keyword arguments for matplotlib.pyplot.hist2d()
     :return: figure handle for the plot
     """
     # Set style. Overrides: kwargs > style input > Style() defaults
     config: Style = Style(**{**Style().dict(), **make_dict(style), **make_dict(kwargs)})
-    kwargs: Dict[str, Any] = {
-        k: v for k, v in kwargs.items() if (k not in config.dict()) or (k == "cmap")
-    }
+    kwargs: Dict[str, Any] = {k: v for k, v in kwargs.items() if (k not in config.dict()) or (k == "cmap")}
     kwargs.setdefault("cmap", config.sequential_cmap)
 
     x_data: List[Any] = to_list_if_other_array(x_data)
     y_data: List[Any] = to_list_if_other_array(y_data)
 
     # Make the plot
-    figure_handle: plt.Figure = plt.figure(
-        facecolor=config.facecolor, figsize=config.figsize
-    )
+    figure_handle: plt.Figure = plt.figure(facecolor=config.facecolor, figsize=config.figsize)
     if zscale.lower() == "log":
         kwargs.setdefault("norm", matplotlib.colors.LogNorm())
     elif zscale.lower() != "linear":
@@ -634,6 +614,8 @@ def visualize_hist2d(
     if show_colorbar:
         cbar: plt.colorbar.Colorbar = plt.colorbar()
         cbar.set_label(colorbar_label, rotation=90, fontsize=config.label_fontsize)
+    if show:
+        plt.show()
     return figure_handle
 
 
@@ -649,6 +631,7 @@ def visualize_surface(
     style: Style = None,
     watermark: str = None,
     y_axis_ascending: bool = True,
+    show: bool = False,
     **kwargs,
 ) -> plt.Figure:
     """Plots a surface (note: seaborn heatmap has somewhat rigid requirements for data completeness)
@@ -664,15 +647,14 @@ def visualize_surface(
     :param style: configuration object (optional)
     :param watermark: watermark text
     :param y_axis_ascending: sort the axis low to high values bottom to top
+    :param show: set to true to trigger plt.show()
     :param kwargs: additional keyword arguments for seaborn heatmap()
     :return: figure handle for the plot
     """
     # Set style. Overrides: kwargs > style input > Style() defaults
     config: Style = Style(**{**Style().dict(), **make_dict(style), **make_dict(kwargs)})
 
-    kwargs: Dict[str, Any] = {
-        k: v for k, v in kwargs.items() if (k not in config.dict()) or (k == "cmap")
-    }
+    kwargs: Dict[str, Any] = {k: v for k, v in kwargs.items() if (k not in config.dict()) or (k == "cmap")}
     kwargs.setdefault("cmap", config.sequential_cmap)
 
     x_data: List[Any] = to_list_if_other_array(x_data)
@@ -684,9 +666,7 @@ def visualize_surface(
     pivoted: pd.DataFrame = df.pivot("y", "x", "z")
 
     # Make the plot
-    figure_handle: plt.Figure = plt.figure(
-        facecolor=config.facecolor, figsize=config.figsize
-    )
+    figure_handle: plt.Figure = plt.figure(facecolor=config.facecolor, figsize=config.figsize)
     ax: plt.Axes = sns.heatmap(pivoted, **kwargs)
 
     # Adjust view & style where applicable
@@ -697,6 +677,8 @@ def visualize_surface(
     apply_watermark(watermark, style=config)
     if y_axis_ascending:
         ax.invert_yaxis()
+    if show:
+        plt.show()
     return figure_handle
 
 
@@ -734,9 +716,7 @@ def visualize_embedded_surface(
     """
     # Set style. Overrides: kwargs > style input > Style() defaults
     config: Style = Style(**{**Style().dict(), **make_dict(style), **make_dict(kwargs)})
-    kwargs: Dict[str, Any] = {
-        k: v for k, v in kwargs.items() if (k not in config.dict()) or (k == "cmap")
-    }
+    kwargs: Dict[str, Any] = {k: v for k, v in kwargs.items() if (k not in config.dict()) or (k == "cmap")}
     kwargs.setdefault("cmap", config.sequential_cmap)
 
     x_data: List[Any] = to_list_if_other_array(x_data)
@@ -744,9 +724,7 @@ def visualize_embedded_surface(
     z_data: List[Any] = to_list_if_other_array(z_data)
 
     # Make the plot
-    figure_handle: plt.Figure = plt.figure(
-        facecolor=config.facecolor, figsize=config.figsize
-    )
+    figure_handle: plt.Figure = plt.figure(facecolor=config.facecolor, figsize=config.figsize)
     ax = Axes3D(figure_handle)
     if log_norm_colors:
         kwargs.setdefault("norm", matplotlib.colors.LogNorm())
@@ -774,9 +752,7 @@ def surface_from_dataframe(
     """Extremely thin wrapper around plot_surface() that extracts the data from a dataframe"""
     col_names: List[str] = [x_col_name, y_col_name, z_col_name]
     if any(element not in (keys := df.keys().tolist()) for element in col_names):
-        raise ValueError(
-            f"Could not find all of {col_names} in data frame keys: {keys}"
-        )
+        raise ValueError(f"Could not find all of {col_names} in data frame keys: {keys}")
     return visualize_surface(df[x_col_name], df[y_col_name], df[z_col_name], **kwargs)
 
 
